/*
  This is an example of how to create a frame delay effect using JavaScript in PixelConduit.
  Rendering is GPU-accelerated, so this script renders essentially as fast as native code.

  To use this script, create a Scripted Effect node widget, click on "Edit Script",
  paste this code into the Script Editor window, and click "Save Script".

  Script written by Pauli Ojala (email: pauli -at- lacquer -dot- fi).
  You may modify, reuse and redistribute this code without limitations.
*/


// initial value for delay.
this.numDelayFrames = 8;

// array used to store frames that have been delayed.
// will be filled out at runtime with Surface objects (= GPU frame buffers).
this.delayedFrames = [];


// this defines the user interface for this node widget in PixelConduit.
this.nodeUIDefinition = [
  { "type": "numberinput", "id": "delay",
	"label": "Delay in frames", "color": "#ccc", "width": "300",
	"min": 1, "max": 15, "increment": 1,
	"actionBinding": "this.onDelayValueChange"
	}
];

this.onDelayValueChange = function(controlId) {
	// this function was specified as the 'actionBinding' in the node UI definition above.
	// we'll just get the current value from the control, round it to an integer, and save it.
    var control = this.getUI().getChildById(controlId);

    var n = Math.floor(control.numberValue);
	this.numDelayFrames = Math.min(15, n);
	
	// if the delay is shorter than before, remove any extra frames
	if (this.delayedFrames.length > this.numDelayFrames) {
		this.delayedFrames.splice(this.numDelayFrames, this.delayedFrames.length - this.numDelayFrames);
	}
	
    return true;  // returning 'true' signals Conduit that it needs to render after this event
}

this.onCreateUI = function() {
	// called when UI is created.
	// set up the default values here to match our script state.
	this.getUI().getChildById("delay").numberValue = this.numDelayFrames;
}


this.onRender = function(inTex1, inTex2, inTex3, inTex4, inMap) {
    var outSurf = this.outputSurface;
	var cacheSurf;

    if ( !inTex1) {  // if we have no input, just bail
        outSurf.clear();
        return;
    }

	// utility function for comparing surface sizes
	function isSameSize(a, b) {
		if ( !a || !b || !a.width || !b.width)
			return false;
		return ((a.width == b.width) && (a.height == b.height));
	}

	if (this.delayedFrames.length > 0) {
		// has our output size changed?
		if ( !isSameSize(outSurf, this.delayedFrames[0])) {
			// if so, play it safe by clearing out the cached frames.
			this.delayedFrames = [];
		}
	}

	if (this.delayedFrames.length < this.numDelayFrames) {
		// we don't have enough delayed frames stored yet, so just clear the output
		// and create a new surface for caching the current input frame.
		outSurf.clear();
				
		cacheSurf = new Surface(outSurf.width, outSurf.height);
	}
	else {
		// we have filled our buffer of delayed frames.
		// copy the oldest one into the output, and recycle that surface for caching the current input frame.
		
		// get the oldest frame, and remove the frame from the array.
		// (for a complicated reason related to how JS garbage collection works,
		// it's not actually safe to set 'delayedSurf' from the return value of the splice call, even though it should be legal...)
		var delayedSurf = this.delayedFrames[0];
		this.delayedFrames.splice(0, 1);
		
		outSurf.drawingContext.textureArray = [ delayedSurf.texture ];
		outSurf.drawWithinBounds();  // this draws into the surface using the current state of the drawingContext

		cacheSurf = delayedSurf;
	}
	
	// copy the input image into the cache surface.
	// specify linear sampling just in case the input size differs, so we don't get the ugly jaggies.
	inTex1.samplingMode = 'linear';
	cacheSurf.drawingContext.textureArray = [ inTex1 ];
	cacheSurf.drawWithinBounds();
	cacheSurf.drawingContext.textureArray = [];

	// the cache surface now contains the latest input image.
	// store it in our array of delayed frames.
	var n = this.delayedFrames.push(cacheSurf);
}
