/*
 This effect example renders with GPU acceleration using Conduit's Surface API.
 
 The input image is rendered 16 times into a 3D layer, and a "texture accumulator"
 is used to combine those render passes into a single floating-point image.
 
 Accumulators can be used for effects like motion blur, antialiasing, soft shadows...
 
 In this case, we modify the projection matrix for each pass so that the resulting
 image has a jitter effect. It looks like camera shake.
 
 For demonstration purposes, a shader is also used to tint the image to yellow.
 

 To use this script, create a Scripted Effect node widget, click on "Edit Script",
 paste this code into the Script Editor window, and click "Save Script".
 
 Script written by Pauli Ojala (email: pauli -at- lacquer -dot- fi).
 You may modify, reuse and redistribute this code without limitations.
 */

this.rotX = 0;
this.rotY = 0;
this.rotZ = 0;
this.fov = 50;
this.jitterAmount = 0.1;
this.yellowTint = 0.3;

this.numSamples = 16;
this.sampleOffsetsX = [];
this.sampleOffsetsY = [];

// create initial sample offsets
for (var i = 0; i < this.numSamples; i++) {
    this.sampleOffsetsX.push(80*(Math.random() - 0.5));
    this.sampleOffsetsY.push(80*(Math.random() - 0.5));
}

this.clickCursorIndex = 0;


this.nodeUIDefinition = [
    { "type": "numberinput", "id": "xrot", "label": "Rotate X", "color": "white", "min": -180, "max": 180, "actionBinding": "this.onRotateX" },
    { "type": "numberinput", "id": "yrot", "label": "Rotate Y", "color": "white", "min": -180, "max": 180, "actionBinding": "this.onRotateY" },
    { "type": "numberinput", "id": "zrot", "label": "Rotate Z", "color": "white", "min": -180, "max": 180, "actionBinding": "this.onRotateZ" },
    { "type": "numberinput", "id": "fov", "label": "Zoom", "color": "white", "min": 10, "max": 120, "actionBinding": "this.onFOV" },
    { "type": "numberinput", "id": "jitter", "label": "Jitter amount", "color": "white", "min": 0, "max": 1, "actionBinding": "this.onSetJitter" },
    { "type": "numberinput", "id": "yellow", "label": "Yellow tint", "color": "white", "min": 0, "max": 1, "actionBinding": "this.onSetYellowTint" },
    { "type": "canvas", "id": "canvas1", "width": 220, "height": 185, "actionBinding": "this.onCanvasEvent" }
];

this.onCreateUI = function() {
	this.getUI().getChildById("fov").numberValue = this.fov;
	this.getUI().getChildById("yellow").numberValue = this.yellowTint;
    this.getUI().getChildById("jitter").numberValue = this.jitterAmount;
	this.refreshCanvasDisplay();
}

this.onRotateX = function(controlId) {
    var control = this.getUI().getChildById(controlId);
    this.rotX = control.numberValue;
    return true;
}
this.onRotateY = function(controlId) {
    var control = this.getUI().getChildById(controlId);
    this.rotY = control.numberValue;
    return true;
}
this.onRotateZ = function(controlId) {
    var control = this.getUI().getChildById(controlId);
    this.rotZ = control.numberValue;
    return true;
}
this.onFOV = function(controlId) {
    var control = this.getUI().getChildById(controlId);
    this.fov = control.numberValue;
    return true;
}
this.onSetJitter = function(controlId) {
    var control = this.getUI().getChildById(controlId);
    this.jitterAmount = control.numberValue;
    return true;
}
this.onSetYellowTint = function(controlId) {
    var control = this.getUI().getChildById(controlId);
    this.yellowTint = control.numberValue;
    return true;
}
this.onCanvasEvent = function(controlId, event) {
    var canvas = this.getUI().getChildById(controlId);
    var evType = event.eventType;
    
    if (evType === "mousedown" || evType === "mousedrag") {
        var x = event.viewX;
        var y = event.viewY;
        
        // check if the click is within the grey box
        x -= 10;
        y -= 30;
        var w = 150;
        if (x >= 0 && y >= 0 && x < w && y < w) {
            var xoff = x - w/2;
            var yoff = y - w/2;
            
            this.sampleOffsetsX[this.clickCursorIndex] = xoff;
            this.sampleOffsetsY[this.clickCursorIndex] = yoff;
            
            this.clickCursorIndex++;
            if (this.clickCursorIndex == this.numSamples)
                this.clickCursorIndex = 0;
            
            this.refreshCanvasDisplay();
        }
    }
    
    return true;
}

this.refreshCanvasDisplay = function() {
    var canvas = this.getUI().getChildById("canvas1");
    var ctx = canvas.getContext("2d");
    
    ctx.save();
    
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    
    labelLine1 = "Click and drag in the grey box to modify";
    labelLine2 = "sampling offsets.";
    
    ctx.fillStyle = "#ccc";
    ctx.font = "bold 9px LucidaGrande";
    ctx.fillText(labelLine1, 4, 10);
    ctx.fillText(labelLine2, 4, 22);
    
    var w = 150;
    
    ctx.translate(10, 30);
    
    ctx.fillStyle = "#444";
    ctx.fillRect(0, 0, w, w);
    ctx.strokeStyle = "#222";
    ctx.lineWidth = 0.7;
    ctx.strokeRect(-0.5, -0.5, w+1, w+1);
    
    ctx.strokeStyle = "#fff";
    ctx.lineWidth = 0.8;
    
    for (var i = 0; i < this.numSamples; i++) {
        var pointX = w/2 + this.sampleOffsetsX[i];
        var pointY = w/2 + this.sampleOffsetsY[i];
        var pointSize = 4;
        
        ctx.beginPath();
        ctx.moveTo(pointX - pointSize, pointY - pointSize);
        ctx.lineTo(pointX + pointSize, pointY + pointSize);
        ctx.moveTo(pointX - pointSize, pointY + pointSize);
        ctx.lineTo(pointX + pointSize, pointY - pointSize);
        ctx.stroke();
    }
    ctx.restore();
}


Math.DEG2RAD = Math.PI / 180;
Math.RAD2DEG = 180 / Math.PI;

this.draw3DLayer = function(surf, tex, colorArray, sampleIndex) {
	if ( !tex) {
		surf.clear();
		return;
	}
	tex.samplingMode = "linear";
	surf.drawingContext.textureArray = [ tex ];
	
	// A shader is a pixel-processing program that executes directly on the GPU.
	// This shader simply tints the source image by multiplying each pixel with the given color value.
	// The shader program is written in the "OpenGL fragment program assembly" format.
	// (Conduit supports this format on all platforms, even Windows).
    //
    // Instead of a shader, we could use a Conduit effect (designed graphically in the Conduit Editor).
    // For anything more complex, you'd probably want to do that rather than write a raw shader --
    // this shader code is thus mainly for demonstration purposes.
	var shader = this.colorShader
    || new Shader("!!ARBfp1.0"+
                  "TEMP t1;"+
                  "TEX t1, fragment.texcoord[0], texture[0], RECT;"+
                  "MUL t1, t1, program.local[0];"+
                  "MOV result.color, t1;"+
                  "END");
	this.colorShader = shader;
	surf.drawingContext.shader = shader;
	surf.drawingContext.setShaderParam(0, colorArray);
    
	// Create a perspective projection matrix.
	var projTrs = new Transform3D();
	var aspectRatio = surf.width / surf.height;
	var fnear = 9;
	var ffar = 0.001;
	projTrs.perspective(this.fov * Math.DEG2RAD, aspectRatio, fnear, ffar);
	
	// To create a jitter in the view plane, modify the projection matrix.
	var m = projTrs.matrixArray;
	var xoff = this.sampleOffsetsX[sampleIndex] + this.jitterAmount*(160*(Math.random() - 0.5));
	var yoff = this.sampleOffsetsY[sampleIndex] + this.jitterAmount*(160*(Math.random() - 0.5));
	m[3] = xoff / surf.width;
	m[7] = yoff / surf.height;
	projTrs.matrixArray = m;
	
	// Next we'll set the model-view matrix.
	// It maps object 3D coordinates to "view" coordinates; the camera position is applied to the same matrix.
	var mvTrs = new Transform3D();
	var cameraZ = 2;
	var centerZ = 1;
	mvTrs.rotate(this.rotX * Math.DEG2RAD, 1, 0, 0);
	mvTrs.rotate(this.rotY * Math.DEG2RAD, 0, 1, 0);
	mvTrs.rotate(this.rotZ * Math.DEG2RAD, 0, 0, 1);
	mvTrs.lookAt(0, 0, cameraZ,
	             0, 0, centerZ,
	             0, 1, 0);
    
	surf.drawingContext.projectionTransform = projTrs;
	surf.drawingContext.modelViewTransform = mvTrs;
	
	surf.clear();
	
	var w = 1.5;
	var h = 1.0;
	surf.draw2DVertices("triangle-fan", [ { "x": -w/2, "y": -h/2, "u": 0.0, "v": 0.0 },
                                         { "x":  w/2, "y": -h/2, "u": 1.0, "v": 0.0 },
                                         { "x":  w/2, "y":  h/2, "u": 1.0, "v": 1.0 },
                                         { "x": -w/2, "y":  h/2, "u": 0.0, "v": 1.0 },
                                         ]);
}

this.onRender = function(inTex1, inTex2, inTex3, inTex4, inMap) {
    var tempSurf = this.getTempSurface();
    var tintColor = [1, 1, (1 - this.yellowTint), 1];
    
    var acc = this.acc || new TextureAccumulator(720, 480);
    this.acc = acc;
    
    acc.numberOfSamples = this.numSamples;
    acc.start();
    
    for (var i = 0; i < acc.numberOfSamples; i++) {
        this.draw3DLayer(tempSurf, inTex1, tintColor, i);
        acc.addTexture(tempSurf.texture);
    }
    
    acc.finishIntoSurface(this.outputSurface);
}