/*
 This is an example of a fairly complex custom user interface
 created by a script in the Project window.
 
 This same code is available in the Starting Point window's Templates
 under the name "Movie Deck". You might want to look there first for an
 example of how to use this script in practice.
 
 
 To use this script, create a Script Widget node widget, click on "Edit Script",
 paste this code into the Script Editor window, and click "Save Script".
 
 Script written by Pauli Ojala (email: pauli -at- lacquer -dot- fi).
 You may modify, reuse and redistribute this code without limitations.
 */


this.nodeUIDefinition = [
  { "type": "canvas", "id": "canvas1", "actionBinding": "this.onUICanvasEvent", "width": 250, "height": 180 }
];

this.deckItemW = 60;
this.deckItemH = 40;
this.deckItemsPerRow = 4;
this.deckNumRows = 3;
this.topMargin = 10;

this.mediaRefs = [];
this.activeMediaRef = null;

this.outputImage = null;
this.outputCanvas = null;

// When this script is loaded, check if we have previously stored media URLs
// in our "persistent data" object.
if (this.persistentData) {
    for (var i = 0; i < this.deckItemsPerRow * this.deckNumRows; i++) {
        var url = this.persistentData[i];
        if (url && url.length > 0) {
            this.mediaRefs[i] = new MediaRef(url);
        }
    }
}

this.onRender = function() {
    // The image returned here contains info about the currently playing clip.
    // We only need to update the image when a new clip is activated,
    // so it's done in a separate updateOutputImage() function rather than here.
	return [ this.outputImage ];
}

// This function name was specified in "nodeUIDefinition" above.
this.onUICanvasEvent = function(canvasId, event) {
    var canvas = this.getUI().getChildById(canvasId);
    var evType = event.eventType;
    
    var clickedItem = -1;
    if (evType === "mousedown" || evType === "mouseup") {
        var clickedItemX = Math.floor(event.viewX / this.deckItemW);
        var clickedItemY = Math.floor((event.viewY - this.topMargin) / this.deckItemH);
        
        if (clickedItemX >= this.deckItemsPerRow || clickedItemX < 0
            || clickedItemY >= this.deckNumRows || clickedItemY < 0) {
            return;
        }
        clickedItem = clickedItemY * this.deckItemsPerRow + clickedItemX;
        
        // is this a double-click?
        if (evType === "mouseup" && event.clickCount == 2) {
            var mediaRef = stream.browseForMediaRef();
            
            this.mediaRefs[clickedItem] = mediaRef;
            
            // Store the media URLs in our persistent storage, so they're not lost if this script widget is reset.
            var mediaURLs = [];
            for (var i = 0; i < this.deckItemsPerRow * this.deckNumRows; i++) {
                var ref = this.mediaRefs[i];
                mediaURLs[i] = (ref) ? ref.src : "";
            }
            this.persistentData = mediaURLs;
            
            this.drawUICanvasContents();
        }
        // is this a single-click?
        else if (evType === "mousedown" && event.clickCount == 1) {
            var mediaRef = this.mediaRefs[clickedItem];
            if (mediaRef) {
                stream.getNodeById("source").mediaRef = mediaRef;
                
                this.activeMediaRef = mediaRef;
                this.updateOutputImage();
            }
        }
    }
    return true;
}

this.drawUICanvasContents = function() {
    var canvas = this.getUI().getChildById("canvas1");
    var ctx = canvas.getContext("2d");
    
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    
    ctx.font = "bold 11px Helvetica";
    ctx.strokeStyle = "rgba(255, 255, 255, 0.8)";
    ctx.lineWidth = 0.8;
    
    for (var y = 0; y < this.deckNumRows; y++) {
        for (var x = 0; x < this.deckItemsPerRow; x++) {
            var itemIndex = y*this.deckItemsPerRow + x;
            var mediaRef = this.mediaRefs[itemIndex];
            var itemX = x*this.deckItemW;
            var itemY = y*this.deckItemH + this.topMargin;
            
            if (mediaRef) {
                var poster = mediaRef.getPosterImage();
                if (poster) {
                    ctx.drawImage(poster, itemX, itemY, this.deckItemW-1, this.deckItemH-1);
                }
            }
            var labelText = ""+(itemIndex+1);
            
            ctx.fillStyle = "#000";
            ctx.fillText(labelText, itemX + 11, itemY + 21);
            ctx.fillStyle = "#dc0";
            ctx.fillText(labelText, itemX + 10, itemY + 20);
            
            ctx.strokeRect(itemX+0.5, itemY+0.5, this.deckItemW-2, this.deckItemH-2);
        }
    }
    
    ctx.font = "11px LucidaGrande";
    ctx.fillStyle = "#fff";
    
    ctx.fillText("Double-click to load a movie or image.", 8, this.deckNumRows * this.deckItemH + 30);
    ctx.fillText("Single-click to activate the loaded clip.", 8, this.deckNumRows * this.deckItemH + 45);
}

// This function is called when Conduit creates the project UI for our node.
this.onCreateUI = function() {
	this.drawUICanvasContents();
	this.updateOutputImage();
}

this.updateOutputImage = function() {
    if ( !this.outputCanvas) {
        // only create the canvas object once
        this.outputCanvas = new Canvas(960, 540);
    }
    var ctx = this.outputCanvas.getContext("2d");
    
    ctx.clearRect(0, 0, this.outputCanvas.width, this.outputCanvas.height);
    
    ctx.font = "bold 25px Helvetica";
    ctx.fillStyle = "#fff";
    
    var mediaRef = this.activeMediaRef;
    var text;
    if ( !mediaRef) {
        text = "";
    } else {
        // get the last component from the media URL; it should be the file name (e.g. "example.mov")
        var url = mediaRef.src;
        var lastComp = (url.lastIndexOf("/") != -1) ? url.slice(url.lastIndexOf("/") + 1) : "";
        
        var duration = Math.ceil(mediaRef.duration * 100) / 100;
        text = "Playing: "+lastComp+"    Duration: "+duration+" s";
    }
    
    ctx.fillText(text, 20, 40, this.outputCanvas.width-40, this.outputCanvas.height-60);
    
    this.outputImage = new Image(this.outputCanvas);
}
